﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Markup;
using HEAL.Attic;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableType("D42922EF-3634-4750-9D3E-B3C327D005F5")]
  public class DoubleVector : Vector<double> {

    public DoubleVector(IEnumerable<double> values)
      : base(values) { }


    [StorableConstructor]
    protected DoubleVector(StorableConstructorFlag _)
      : base(_) { }

    public static DoubleVector operator +(DoubleVector lhs, DoubleVector rhs) {
      return new DoubleVector(VecCalc(lhs, rhs, (l, r) => l + r));
    }
    public static DoubleVector operator -(DoubleVector lhs, DoubleVector rhs) {
      return new DoubleVector(VecCalc(lhs, rhs, (l, r) => l - r));
    }
    public static DoubleVector operator *(DoubleVector lhs, DoubleVector rhs) {
      return new DoubleVector(VecCalc(lhs, rhs, (l, r) => l * r));
    }
    public static DoubleVector operator /(DoubleVector lhs, DoubleVector rhs) {
      return new DoubleVector(VecCalc(lhs, rhs, (l, r) => l / r));
    }
    private static IEnumerable<double> VecCalc(IReadOnlyList<double> lhs, IReadOnlyList<double> rhs, Func<double, double, double> resultFunc) {
      if (lhs.Count != rhs.Count) throw new InvalidOperationException("Vectors are not of equal length.");
      return lhs.Zip(rhs, resultFunc);
    }

    public static DoubleVector operator +(DoubleVector lhs, double rhs) {
      return new DoubleVector(lhs.Select(l => l + rhs));
    }
    public static DoubleVector operator -(DoubleVector lhs, double rhs) {
      return new DoubleVector(lhs.Select(l => l - rhs));
    }
    public static DoubleVector operator *(DoubleVector lhs, double rhs) {
      return new DoubleVector(lhs.Select(l => l * rhs));
    }
    public static DoubleVector operator /(DoubleVector lhs, double rhs) {
      return new DoubleVector(lhs.Select(l => l / rhs));
    }

    public double Sum() {
      return values.Sum();
    }

    public double Average() {
      return values.Average();
    }
  }
}